;################################################################## RCVBIN.ASL
; Purpose.....: Receive data in binary format over USART on EOM-MPF and
;		store at addresses entered.
;		Serial parameters are 4800,N,8,1.
; Target......: MPF-1P
; Assembler...: ASL V1.42 Beta
; Command Line:	ASL -listline-prefix %04a -L -U -olist <name>.prn <name>.asl
; PsPadParser.: > > > %F(%L):%C: error:
;#############################################################################

;********************************************************************* history
;=========================================================== V01.00:29.08.2024
; Created
;***************************************************************** end history

;***************************************************************** definitions
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++ assembler directives
		CPU	Z80		; use Z80 CPU
		PAGE	0		; no formfeeds in listing file
		
;+++++++++++++++++++++++++++++++++++ hardware, monitor and library definitions
		LISTING	OFF
		INCLUDE	../LIBRARY/MPF1P.DEF
		LISTING	ON
		
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ build addresses
wRBCS:		EQU	0B000H		; code (B000-B14B if IOM-ROM)
wRBDS:		EQU	0FD00H		; data on top of MPF RAM

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ misc
wRBSTAD:	EQU	0F000H		; default store address

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ variables
wRBVS:		EQU	wRBDS		; variables start address
vRBFA:		EQU	wRBVS		; from address for success display
vRBDB:		EQU	vRBFA + 2	; display buffer for address input
wRBVE:		EQU	vRBDB + dbMCPL	; variables end address

;======================================== check overlap with library variables
		IF	wRBVE > dwLVST && MomPass = 2
		WARNING	"Variables overlap with library:\{wRBVE} >> \{dwLVST}"
		ENDIF
;************************************************************* end definitions

;************************************************************** implementation
		ASEG
		ORG	wRBCS
		
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ main program
;======================================================================== RB()
; Purpose.....: Main program
; Labels......: R)eceive B)inary
;=============================================================================
RB:		CALL	dfLLS		; initialize library variables

		LD	HL,0
		LD	(vRBFA),HL	; clear start address

		LD	IX,tRBSO	; sign-on message
		LD	B,030H		; 30H ~1s
		CALL	dfLDT		; show it

;----------------------------------------------------------------- setup USART
		LD	B,dbUB4K	; baudrate 4800
		LD	C,dbU8N1	; 8 data, no parity, 1 stop	
		CALL	dfLUS		; setup USART

;--------------------------------------------- get store address from keyboard
RBGSTAD:	LD	HL,wRBTCPS	; HL= text start address
		LD	DE,vRBDB	; DE= display buffer address
		LD	BC,bRBTCPZ	; BC= text length
		LDIR			; copy text to RAM buffer

		LD	IX,vRBDB	; IX= text buffer address	
		LD	A,2		; set hex mode
		LD	B,4		; use four digits
		LD	C,bRBAGCP	; cursor position
		SCF			; buffer clear

RBGSTADL:	CALL	dfLDKR		; call editor
		LD	IX,0		; set continue mode
		JP	C,RBQ		; aborted with SHIFT&CONTROL
		JR	NZ,RBGSTADL	; not CR, continue reading

		LD	A,D		; DE has inputted value
		OR	E
		JR	NZ,RBW		; if not null, use it
		LD	DE,wRBSTAD	; if null, use default
		
;---------------------------------------------------------------- wait pattern
RBW:		LD	A,':'
		CALL	RBDP		; display wait pattern

		EX	DE,HL		; HL= RAM address
		LD	(vRBFA),HL	; store for end display	
		LD	B,0		; no timeout until 1st byte
		SCF			; set RTS
		CALL	dfLURTS		; active

;-------------------------------------------------------- read byte from USART
RBRL:		CALL	dfLKSAC		; check for abort request
		JR	C,RBQ		; shift and control pressed, exit

		PUSH	BC		; dfLURB returns status in B
		CALL	dfLURB		; get byte from USART
		POP	BC		; recall B
		JR	C,RBER		; CY=1: some error
		JR	NZ,RBRR		; Z=0: character ready

		BIT	0,B		; no byte, timeout check required?
		JR	Z,RBRL		; no, continue waiting for 1st byte

;---------------------------------------------------- check inter-byte timeout
		INC	DE 		; bump timeout counter
		LD	A,D		; A= hibyte of timeout count
		CP	02H		; timeout reached?
		JR	Z,RBDONE	; timeout, transfer done
		JR	RBRL		; not yet, continue waiting
						
;--------------------------------------- byte received, show receiving pattern
RBRR:		LD	DE,0		; clear timeout counter
		SET	0,B		; check timeout from now on
		BIT	1,B		; got byte, receiving pattern shown?
		JR	NZ,RBRS		; shown, continue with store
		SET	1,B		; set shown flag
		LD	C,A		; not yet, show it
		XOR	A		; set RTS
		CALL	dfLURTS		; inactive
		LD	A,'-'
		CALL	RBDP		; display receiving pattern
		SCF			; set RTS
		CALL	dfLURTS		; active again
		LD	A,C
										
;---------------------------- store byte and check if equal and not system RAM
RBRS:		LD	(HL),A		; store in RAM
		LD	C,(HL)		; C= stored byte
		CP	C		; same?
		JR	NZ,RBEV		; not same, no RAM or ROM
		CALL	RBCKM		; system RAM overwrite?
		JR	C,RBEY		; yes, exit with error
		INC	HL		; same, bump RAM pointer
		JR	RBRL		; wait for next byte

;-------------------------------------- success, display from and to addresses
RBDONE:		PUSH	HL		; save last written address
		CALL	dfMCLR		; clear display buffer
		LD	HL,tRBFP	; from prompt
		CALL	dfMMSG		; to display buffer
		LD	HL,(vRBFA)	; HL= from address
		CALL	dfMB4H		; to display buffer
		LD	HL,tRBTP	; to prompt
		CALL	dfMMSG		; to display buffer
		POP	HL		; recall last written address
		CALL	dfMB4H		; to display buffer
		JR	RBX		; display, wait for key

;--------------------------------------------------------------- error handler
RBEV:		LD	DE,tRBEV	; verify error
		JR	RBMSG
RBEY:		LD	DE,tRBEY	; sys RAM overwrite attempt
		JR	RBMSG
RBER:		LD	DE,tRBER	; USART error

RBMSG:		CALL	dfMCLR		; clear display buffer
		CALL	dfMB4H		; last written address to display buf		
		EX	DE,HL		; HL= error message
  		CALL	dfMMSG		; error message to display buffer

;---------------------------------- display exit message and return to monitor
RBX:		LD	IX,dvMDB	; IX= display buffer address
		CALL	dfMSC		; print message and wait for key

RBQ:		XOR	A
		CALL	dfLURTS		; disable handshake
		JP	dwMRET		; monitor warm start

;-------------------------------------------------------------- text constants
tRBSO:		DB	'    RB V24-08-29    '
tRBFP:		DB	'FROM ',                0DH
tRBTP:		DB	'  TO ',                0DH
tRBEV:		DB	' VERIFY ERROR',        0DH
tRBEY:		DB	' SYS-RAM OVR',         0DH
tRBER:		DB	' USART ERROR',         0DH

;------------------------------------------ text constants to be copied to RAM
wRBTCPS:	EQU	$
tRBAG:		DB	'ADDRESS             ', 0DH
bRBAGCP:	EQU	8	
wRBTCPE:	EQU	$
bRBTCPZ:	EQU	wRBTCPE-wRBTCPS
;==================================================================== eoM RB()
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end main program

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ subroutines
;===================================================================== RBCKM()
; Purpose.....: Check if system RAM is going to be overwitten
; Input.......: HL: current write address
; Output......: CY: 0= no overwite
;		    1= overwrite will occur
; Registers...: AF destroyed
; Labels......: R)ead B)in C)ecK) M)emory
;=============================================================================
RBCKM:		PUSH	HL
		PUSH	BC

		LD	BC,dwMRAE	; BC= MPF RAM end - system use
		SCF			; set CY for SBC
		SBC	HL,BC		; CY=1 if BC > HL
		CCF			; CY=1 if BC <= HL

		POP	BC
		POP	HL
		RET
;================================================================= eoF RBCKM()

;====================================================================== RBDP()
; Purpose.....:	Display static pattern in the center of the display to
;		signal progress.
; Input.......: A: Pattern character to display
; Registers...:	AF destroyed
; Labels......: R)ead B)in D)isplay P)attern
;=============================================================================
RBDP:		PUSH	DE
		PUSH	BC

		LD	D,00011111B	; digits 8-16
		LD	E,10000000B	; digits 1-8
		LD	C,00000000B	; digits 17-20
		LD	B,A		; save pattern
		CALL	dfLDP		; display the pattern

		POP	BC
		POP	DE
		RET
;================================================================== eoF RBDP()
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end subroutines

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ size info
wRBCSE:		EQU	$

		IF	MomPass = 2
		MESSAGE "Code from \{wRBCS} to \{wRBCSE} Size \{wRBCSE-wRBCS}"
		MESSAGE "Data from \{wRBDS} to \{wRBVE} Size \{wRBVE-wRBDS}"
		MESSAGE "     remaining \{dwLVST-wRBVE}"
		ENDIF
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end size info

;############################################################## EOF RCVBIN.ASL
		END
